package com.genersoft.iot.vmp.vmanager.gb28181.play;

import com.alibaba.fastjson.JSONArray;
import com.genersoft.iot.vmp.common.StreamInfo;
import com.genersoft.iot.vmp.gb28181.bean.SsrcTransaction;
import com.genersoft.iot.vmp.gb28181.session.VideoStreamSessionManager;
import com.genersoft.iot.vmp.gb28181.bean.Device;
import com.genersoft.iot.vmp.gb28181.transmit.callback.DeferredResultHolder;
import com.genersoft.iot.vmp.gb28181.transmit.callback.RequestMessage;
import com.genersoft.iot.vmp.media.zlm.ZLMRESTfulUtils;
import com.genersoft.iot.vmp.media.zlm.dto.MediaServerItem;
import com.genersoft.iot.vmp.service.IMediaServerService;
import com.genersoft.iot.vmp.storager.IRedisCatchStorage;
import com.genersoft.iot.vmp.vmanager.bean.AudioBroadcastResult;
import com.genersoft.iot.vmp.vmanager.bean.WVPResult;
import com.genersoft.iot.vmp.vmanager.gb28181.play.bean.PlayResult;
import com.genersoft.iot.vmp.service.IMediaService;
import com.genersoft.iot.vmp.service.IPlayService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONObject;
import com.genersoft.iot.vmp.gb28181.transmit.cmd.impl.SIPCommander;
import com.genersoft.iot.vmp.storager.IVideoManagerStorage;
import org.springframework.web.context.request.async.DeferredResult;

import java.util.List;
import java.util.UUID;

/**
 * @author lin
 */
@Api(tags = "国标设备点播")
@CrossOrigin
@RestController
@RequestMapping("/api/play")
public class PlayController {

	private final static Logger logger = LoggerFactory.getLogger(PlayController.class);

	@Autowired
	private SIPCommander cmder;

	@Autowired
	private VideoStreamSessionManager streamSession;

	@Autowired
	private IVideoManagerStorage storager;

	@Autowired
	private IRedisCatchStorage redisCatchStorage;

	@Autowired
	private ZLMRESTfulUtils zlmresTfulUtils;

	@Autowired
	private DeferredResultHolder resultHolder;

	@Autowired
	private IPlayService playService;

	@Autowired
	private IMediaService mediaService;

	@Autowired
	private IMediaServerService mediaServerService;

	@ApiOperation("开始点播")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "deviceId", value = "设备ID", dataTypeClass = String.class),
			@ApiImplicitParam(name = "channelId", value = "通道ID", dataTypeClass = String.class),
	})
	@GetMapping("/start/{deviceId}/{channelId}")
	public DeferredResult<ResponseEntity<String>> play(@PathVariable String deviceId,
													   @PathVariable String channelId) {

		// 获取可用的zlm
		Device device = storager.queryVideoDevice(deviceId);
		MediaServerItem newMediaServerItem = playService.getNewMediaServerItem(device);
		PlayResult playResult = playService.play(newMediaServerItem, deviceId, channelId, null, null, null);

		return playResult.getResult();
	}

	@ApiOperation("停止点播")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "deviceId", value = "设备ID", dataTypeClass = String.class),
			@ApiImplicitParam(name = "channelId", value = "通道ID", dataTypeClass = String.class),
	})
	@GetMapping("/stop/{deviceId}/{channelId}")
	public DeferredResult<ResponseEntity<String>> playStop(@PathVariable String deviceId, @PathVariable String channelId) {

		logger.debug(String.format("设备预览/回放停止API调用，streamId：%s_%s", deviceId, channelId ));

		String uuid = UUID.randomUUID().toString();
		DeferredResult<ResponseEntity<String>> result = new DeferredResult<>();

		// 录像查询以channelId作为deviceId查询
		String key = DeferredResultHolder.CALLBACK_CMD_STOP + deviceId + channelId;
		resultHolder.put(key, uuid, result);
		StreamInfo streamInfo = redisCatchStorage.queryPlayByDevice(deviceId, channelId);
		if (streamInfo == null) {
			RequestMessage msg = new RequestMessage();
			msg.setId(uuid);
			msg.setKey(key);
			msg.setData("点播未找到");
			resultHolder.invokeAllResult(msg);
			storager.stopPlay(deviceId, channelId);
			return result;
		}
		cmder.streamByeCmd(deviceId, channelId, streamInfo.getStream(), null, eventResult -> {
			redisCatchStorage.stopPlay(streamInfo);
			storager.stopPlay(streamInfo.getDeviceID(), streamInfo.getChannelId());
			RequestMessage msgForSuccess = new RequestMessage();
			msgForSuccess.setId(uuid);
			msgForSuccess.setKey(key);
			msgForSuccess.setData("success");
			resultHolder.invokeAllResult(msgForSuccess);
		});

		if (deviceId != null || channelId != null) {
			JSONObject json = new JSONObject();
			json.put("deviceId", deviceId);
			json.put("channelId", channelId);
			RequestMessage msg = new RequestMessage();
			msg.setId(uuid);
			msg.setKey(key);
			msg.setData(json.toString());
			resultHolder.invokeAllResult(msg);
		} else {
			logger.warn("设备预览/回放停止API调用失败！");
			RequestMessage msg = new RequestMessage();
			msg.setId(uuid);
			msg.setKey(key);
			msg.setData("streamId null");
			resultHolder.invokeAllResult(msg);
		}

		// 超时处理
		result.onTimeout(()->{
			logger.warn(String.format("设备预览/回放停止超时，deviceId/channelId：%s_%s ", deviceId, channelId));
			RequestMessage msg = new RequestMessage();
			msg.setId(uuid);
			msg.setKey(key);
			msg.setData("Timeout");
			resultHolder.invokeAllResult(msg);
		});
		return result;
	}

	/**
	 * 将不是h264的视频通过ffmpeg 转码为h264 + aac
	 * @param streamId 流ID
	 * @return
	 */
	@ApiOperation("将不是h264的视频通过ffmpeg 转码为h264 + aac")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "streamId", value = "视频流ID", dataTypeClass = String.class),
	})
	@PostMapping("/convert/{streamId}")
	public ResponseEntity<String> playConvert(@PathVariable String streamId) {
		StreamInfo streamInfo = redisCatchStorage.queryPlayByStreamId(streamId);
		if (streamInfo == null) {
			streamInfo = redisCatchStorage.queryPlayback(null, null, streamId, null);
		}
		if (streamInfo == null) {
			logger.warn("视频转码API调用失败！, 视频流已经停止!");
			return new ResponseEntity<String>("未找到视频流信息, 视频流可能已经停止", HttpStatus.OK);
		}
		MediaServerItem mediaInfo = mediaServerService.getOne(streamInfo.getMediaServerId());
		JSONObject rtpInfo = zlmresTfulUtils.getRtpInfo(mediaInfo, streamId);
		if (!rtpInfo.getBoolean("exist")) {
			logger.warn("视频转码API调用失败！, 视频流已停止推流!");
			return new ResponseEntity<String>("推流信息在流媒体中不存在, 视频流可能已停止推流", HttpStatus.OK);
		} else {
			String dstUrl = String.format("rtmp://%s:%s/convert/%s", "127.0.0.1", mediaInfo.getRtmpPort(),
					streamId );
			String srcUrl = String.format("rtsp://%s:%s/rtp/%s", "127.0.0.1", mediaInfo.getRtspPort(), streamId);
			JSONObject jsonObject = zlmresTfulUtils.addFFmpegSource(mediaInfo, srcUrl, dstUrl, "1000000", true, false, null);
			logger.info(jsonObject.toJSONString());
			JSONObject result = new JSONObject();
			if (jsonObject != null && jsonObject.getInteger("code") == 0) {
				   result.put("code", 0);
				JSONObject data = jsonObject.getJSONObject("data");
				if (data != null) {
				   	result.put("key", data.getString("key"));
					StreamInfo streamInfoResult = mediaService.getStreamInfoByAppAndStreamWithCheck("convert", streamId, mediaInfo.getId());
					result.put("data", streamInfoResult);
				}
			}else {
				result.put("code", 1);
				result.put("msg", "cover fail");
			}
			return new ResponseEntity<String>( result.toJSONString(), HttpStatus.OK);
		}
	}

	/**
	 * 结束转码
	 * @param key
	 * @return
	 */
	@ApiOperation("结束转码")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "key", value = "视频流key", dataTypeClass = String.class),
	})
	@PostMapping("/convertStop/{key}")
	public ResponseEntity<String> playConvertStop(@PathVariable String key, String mediaServerId) {
		JSONObject result = new JSONObject();
		if (mediaServerId == null) {
			result.put("code", 400);
			result.put("msg", "mediaServerId is null");
			return new ResponseEntity<String>( result.toJSONString(), HttpStatus.BAD_REQUEST);
		}
		MediaServerItem mediaInfo = mediaServerService.getOne(mediaServerId);
		if (mediaInfo == null) {
			result.put("code", 0);
			result.put("msg", "使用的流媒体已经停止运行");
			return new ResponseEntity<String>( result.toJSONString(), HttpStatus.OK);
		}else {
			JSONObject jsonObject = zlmresTfulUtils.delFFmpegSource(mediaInfo, key);
			logger.info(jsonObject.toJSONString());
			if (jsonObject != null && jsonObject.getInteger("code") == 0) {
				result.put("code", 0);
				JSONObject data = jsonObject.getJSONObject("data");
				if (data != null && data.getBoolean("flag")) {
					result.put("code", "0");
					result.put("msg", "success");
				}else {

				}
			}else {
				result.put("code", 1);
				result.put("msg", "delFFmpegSource fail");
			}
			return new ResponseEntity<String>( result.toJSONString(), HttpStatus.OK);
		}


	}

	@ApiOperation("语音广播命令")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "deviceId", value = "设备Id", dataTypeClass = String.class),
			@ApiImplicitParam(name = "channelId", value = "通道Id", dataTypeClass = String.class),
			@ApiImplicitParam(name = "timeout", value = "推流超时时间(秒)", dataTypeClass = Integer.class),
	})
    @GetMapping("/broadcast/{deviceId}/{channelId}")
    @PostMapping("/broadcast/{deviceId}/{channelId}")
    public DeferredResult<WVPResult<AudioBroadcastResult>> broadcastApi(@PathVariable String deviceId, @PathVariable String channelId,  Integer timeout) {
        if (logger.isDebugEnabled()) {
            logger.debug("语音广播API调用");
        }
        Device device = storager.queryVideoDevice(deviceId);
		if (device == null) {
			WVPResult<AudioBroadcastResult> result = new WVPResult<>();
			result.setCode(-1);
			result.setMsg("未找到设备： " + deviceId);
			DeferredResult<WVPResult<AudioBroadcastResult>> deferredResult = new DeferredResult<>();
			deferredResult.setResult(result);
			return deferredResult;
		}
		if (channelId == null) {
			WVPResult<AudioBroadcastResult> result = new WVPResult<>();
			result.setCode(-1);
			result.setMsg("未找到通道： " + channelId);
			DeferredResult<WVPResult<AudioBroadcastResult>> deferredResult = new DeferredResult<>();
			deferredResult.setResult(result);
			return deferredResult;
		}

		String key = DeferredResultHolder.CALLBACK_CMD_BROADCAST + deviceId;
		if (resultHolder.exist(key, null)) {
			WVPResult<AudioBroadcastResult> wvpResult = new WVPResult<>();
			wvpResult.setCode(-1);
			wvpResult.setMsg("设备使用中");
			DeferredResult<WVPResult<AudioBroadcastResult>> deferredResult = new DeferredResult<>();
			deferredResult.setResult(wvpResult);
			return deferredResult;
		}
		if (timeout == null){
			timeout = 30;
		}
		DeferredResult<WVPResult<AudioBroadcastResult>> result = new DeferredResult<>(timeout.longValue()*1000 + 2000);
		String uuid  = UUID.randomUUID().toString();
		result.onTimeout(()->{
			WVPResult<AudioBroadcastResult> wvpResult = new WVPResult<>();
			wvpResult.setCode(-1);
			wvpResult.setMsg("请求超时");
			RequestMessage requestMessage = new RequestMessage();
			requestMessage.setKey(key);
			requestMessage.setData(wvpResult);
			resultHolder.invokeAllResult(requestMessage);
		});
		playService.audioBroadcast(device, channelId, timeout, (msg)->{
			WVPResult<AudioBroadcastResult> wvpResult = new WVPResult<>();
			wvpResult.setCode(-1);
			wvpResult.setMsg(msg);
			RequestMessage requestMessage = new RequestMessage();
			requestMessage.setKey(key);
			requestMessage.setData(wvpResult);
			resultHolder.invokeAllResult(requestMessage);
		});
		resultHolder.put(key, uuid, result);

		return result;
	}


	@ApiOperation("停止语音广播")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "deviceId", value = "设备Id", dataTypeClass = String.class),
			@ApiImplicitParam(name = "channelId", value = "通道Id", dataTypeClass = String.class),
	})
	@GetMapping("/broadcast/stop/{deviceId}/{channelId}")
	@PostMapping("/broadcast/stop/{deviceId}/{channelId}")
	public WVPResult<String> stopBroadcastA(@PathVariable String deviceId, @PathVariable String channelId) {
		if (logger.isDebugEnabled()) {
			logger.debug("停止语音广播API调用");
		}
		playService.stopAudioBroadcast(deviceId, channelId);
		return new WVPResult<>(0, "success", null);
	}

	@ApiOperation("获取所有的ssrc")
	@GetMapping("/ssrc")
	public WVPResult<JSONObject> getSsrc() {
		if (logger.isDebugEnabled()) {
			logger.debug("获取所有的ssrc");
		}
		JSONArray objects = new JSONArray();
		List<SsrcTransaction> allSsrc = streamSession.getAllSsrc();
		for (SsrcTransaction transaction : allSsrc) {
			JSONObject jsonObject = new JSONObject();
			jsonObject.put("deviceId", transaction.getDeviceId());
			jsonObject.put("channelId", transaction.getChannelId());
			jsonObject.put("ssrc", transaction.getSsrc());
			jsonObject.put("streamId", transaction.getStream());
			objects.add(jsonObject);
		}

		WVPResult<JSONObject> result = new WVPResult<>();
		result.setCode(0);
		result.setMsg("success");
		JSONObject jsonObject = new JSONObject();
		jsonObject.put("data", objects);
		jsonObject.put("count", objects.size());
		result.setData(jsonObject);
		return result;
	}

}

